@@ -248,6 +248,16 @@ static Value Uint32OrUndefinedValue(mozilla::Maybe<uint32_t> x) {
   return Uint32Value(x.value());
 }
 
+static Value AsyncEvaluationOrderInt32Value(AsyncEvaluationOrder x) {
+  if (x.isUnset()) {
+    return Int32Value(-1);
+  }
+  if (x.isDone()) {
+    return Int32Value(-2);
+  }
+  return Uint32Value(x.get());
+}
+
 static Value ColumnNumberOneOriginValue(JS::ColumnNumberOneOrigin x) {
   uint32_t column = x.oneOriginValue();
   MOZ_ASSERT(column <= INT32_MAX);
@@ -365,6 +375,12 @@ bool ModuleTypeToString(JSContext* cx, JS::Handle<JSObject*> owner,
     case JS::ModuleType::JSON:
       to.setString(cx->names().json);
       break;
+    case JS::ModuleType::CSS:
+      MOZ_ASSERT_UNREACHABLE("CSS modules are not supported in the shell");
+      break;
+    case JS::ModuleType::Bytes:
+      to.setString(cx->names().bytes);
+      break;
   }
 
   MOZ_ASSERT(!to.isUndefined());
@@ -468,18 +484,14 @@ DEFINE_NATIVE_GETTER_FUNCTIONS(ModuleObject, indirectExportEntries,
                                SpanToArrayFilter<ShellExportEntryWrapper>)
 DEFINE_NATIVE_GETTER_FUNCTIONS(ModuleObject, starExportEntries,
                                SpanToArrayFilter<ShellExportEntryWrapper>)
-DEFINE_GETTER_FUNCTIONS(ModuleObject, maybeDfsIndex, Uint32OrUndefinedValue,
-                        IdentFilter)
 DEFINE_GETTER_FUNCTIONS(ModuleObject, maybeDfsAncestorIndex,
                         Uint32OrUndefinedValue, IdentFilter)
 DEFINE_GETTER_FUNCTIONS(ModuleObject, hasTopLevelAwait, BooleanValue,
                         IdentFilter)
 DEFINE_GETTER_FUNCTIONS(ModuleObject, maybeTopLevelCapability,
                         ObjectOrUndefinedValue, IdentFilter)
-DEFINE_GETTER_FUNCTIONS(ModuleObject, isAsyncEvaluating, BooleanValue,
-                        IdentFilter)
-DEFINE_GETTER_FUNCTIONS(ModuleObject, maybeAsyncEvaluatingPostOrder,
-                        Uint32OrUndefinedValue, IdentFilter)
+DEFINE_GETTER_FUNCTIONS(ModuleObject, asyncEvaluationOrder,
+                        AsyncEvaluationOrderInt32Value, IdentFilter)
 DEFINE_GETTER_FUNCTIONS(ModuleObject, asyncParentModules, ObjectOrNullValue,
                         ListToArrayFilter<ShellModuleObjectWrapper>)
 DEFINE_GETTER_FUNCTIONS(ModuleObject, maybePendingAsyncDependencies,
@@ -499,17 +511,14 @@ static const JSPropertySpec ShellModuleObjectWrapper_accessors[] = {
            ShellModuleObjectWrapper_indirectExportEntriesGetter, 0),
     JS_PSG("starExportEntries",
            ShellModuleObjectWrapper_starExportEntriesGetter, 0),
-    JS_PSG("dfsIndex", ShellModuleObjectWrapper_maybeDfsIndexGetter, 0),
     JS_PSG("dfsAncestorIndex",
            ShellModuleObjectWrapper_maybeDfsAncestorIndexGetter, 0),
     JS_PSG("hasTopLevelAwait", ShellModuleObjectWrapper_hasTopLevelAwaitGetter,
            0),
     JS_PSG("topLevelCapability",
            ShellModuleObjectWrapper_maybeTopLevelCapabilityGetter, 0),
-    JS_PSG("isAsyncEvaluating",
-           ShellModuleObjectWrapper_isAsyncEvaluatingGetter, 0),
-    JS_PSG("asyncEvaluatingPostOrder",
-           ShellModuleObjectWrapper_maybeAsyncEvaluatingPostOrderGetter, 0),
+    JS_PSG("asyncEvaluationOrder",
+           ShellModuleObjectWrapper_asyncEvaluationOrderGetter, 0),
     JS_PSG("asyncParentModules",
            ShellModuleObjectWrapper_asyncParentModulesGetter, 0),
     JS_PSG("pendingAsyncDependencies",
@@ -560,7 +569,28 @@ DEFINE_NATIVE_CREATE(ImportEntry, ShellImportEntryWrapper_accessors, nullptr)
 DEFINE_NATIVE_CREATE(ExportEntry, ShellExportEntryWrapper_accessors, nullptr)
 DEFINE_NATIVE_CREATE(RequestedModule, ShellRequestedModuleWrapper_accessors,
                      nullptr)
-DEFINE_CREATE(ModuleObject, ShellModuleObjectWrapper_accessors, nullptr)
 
 #undef DEFINE_CREATE
 #undef DEFINE_NATIVE_CREATE
+
+JS::ModuleType ShellModuleObjectWrapper::getModuleType() {
+  return static_cast<JS::ModuleType>(getReservedSlot(ModuleTypeSlot).toInt32());
+}
+
+ShellModuleObjectWrapper* ShellModuleObjectWrapper::create(
+    JSContext* cx, JS::Handle<ModuleObject*> target,
+    JS::ModuleType moduleType) {
+  JS::Rooted<JSObject*> obj(cx, JS_NewObject(cx, &class_));
+  if (!obj) {
+    return nullptr;
+  }
+  if (!DefinePropertiesAndFunctions(cx, obj, ShellModuleObjectWrapper_accessors,
+                                    nullptr)) {
+    return nullptr;
+  }
+  auto* wrapper = &obj->as<ShellModuleObjectWrapper>();
+  wrapper->initReservedSlot(TargetSlot, ObjectValue(*target));
+  wrapper->initReservedSlot(ModuleTypeSlot,
+                            Int32Value(static_cast<int32_t>(moduleType)));
+  return wrapper;
+}
